searchNodes=[{"doc":"A tiny extendable DSL evaluator. It's a wrap around Elixir's Code.eval_string/3 or Code.eval_quoted/3 , with the following limitations: No calling module functions; No calling some functions which can cause VM to exit; No sending messages; (optional) memory usage limit; (optional) execution time limit. Here's an example using this module to evaluate a discount number against an order struct: iex&gt; discount_formula = ~s&quot; ...&gt; case order do ...&gt; # old books get a big promotion ...&gt; %{book: %{year: year}} when year &lt; 2000 -&gt; ...&gt; 0.5 ...&gt; ...&gt; %{book: %{tags: tags}} -&gt; ...&gt; # Elixir books! ...&gt; if ~s{elixir} in tags do ...&gt; 0.9 ...&gt; else ...&gt; 1.0 ...&gt; end ...&gt; ...&gt; _ -&gt; ...&gt; 1.0 ...&gt; end ...&gt; &quot; ...&gt; ...&gt; book_order = %{ ...&gt; book : %{ ...&gt; title : &quot;Elixir in Action&quot; , year : 2019 , tags : [ &quot;elixir&quot; ] ...&gt; } ...&gt; } ...&gt; ...&gt; Formular . eval ( discount_formula , [ order : book_order ] ) { :ok , 0.9 } The code being evaluated is just a piece of Elixir code, so it can be expressive when describing business rules. Literals # number iex&gt; Formular . eval ( &quot;1&quot; , [ ] ) { :ok , 1 } # &lt;- note that it&#39;s an integer # plain string iex&gt; Formular . eval ( ~s[&quot;some text&quot;] , [ ] ) { :ok , &quot;some text&quot; } # atom iex&gt; Formular . eval ( &quot;:foo&quot; , [ ] ) { :ok , :foo } # list iex&gt; Formular . eval ( &quot;[:foo, Bar]&quot; , [ ] ) { :ok , [ :foo , Bar ] } # keyword list iex&gt; Formular . eval ( &quot;[a: 1, b: :hi]&quot; , [ ] ) { :ok , [ a : 1 , b : :hi ] } Variables Variables can be passed within the binding parameter. # bound value iex&gt; Formular . eval ( &quot;1 + foo&quot; , [ foo : 42 ] ) { :ok , 43 } Functions in the code Kernel functions and macros Kernel functions and macros are limitedly supported. Only a picked list of them are supported out of the box so that dangerouse functions such as Kernel.exit/1 will not be invoked. Supported functions from Kernel are: [ != : 2 , !== : 2 , * : 2 , + : 1 , + : 2 , ++ : 2 , ++ : 2 , - : 1 , - : 2 , -- : 2 , -- : 2 , / : 2 , &lt; : 2 , &lt;= : 2 , == : 2 , =~ : 2 , &gt; : 2 , &gt;= : 2 , abs : 1 , ceil : 1 , div : 2 , floor : 1 , get_and_update_in : 3 , get_in : 2 , hd : 1 , inspect : 2 , is_atom : 1 , is_binary : 1 , is_bitstring : 1 , is_boolean : 1 , is_float : 1 , is_function : 1 , is_integer : 1 , is_list : 1 , is_map : 1 , is_map_key : 2 , is_number : 1 , is_pid : 1 , is_port : 1 , is_reference : 1 , is_tuple : 1 , length : 1 , map_size : 1 , max : 2 , min : 2 , not : 1 , pop_in : 2 , put_elem : 3 , put_in : 3 , rem : 2 , ... ] Supported macros from Kernel are: [ ! : 1 , &amp;&amp; : 2 , .. : 2 , .. / / : 3 , &lt;&gt; : 2 , and : 2 , get_and_update_in : 2 , if : 2 , in : 2 , is_exception : 1 , is_exception : 2 , is_nil : 1 , is_struct : 1 , is_struct : 2 , or : 2 , pop_in : 1 , put_in : 2 , sigil_C : 2 , sigil_D : 2 , sigil_N : 2 , sigil_R : 2 , sigil_S : 2 , sigil_T : 2 , sigil_U : 2 , sigil_W : 2 , sigil_c : 2 , sigil_r : 2 , sigil_s : 2 , sigil_w : 2 , tap : 2 , then : 2 , to_charlist : 1 , to_string : 1 , unless : 2 , |&gt; : 2 , || : 2 ] Example: # Kernel function iex&gt; Formular . eval ( &quot;min(5, 100)&quot; , [ ] ) { :ok , 5 } iex&gt; Formular . eval ( &quot;max(5, 100)&quot; , [ ] ) { :ok , 100 } Custom functions Custom functions can be provided in two ways, either in a binding lambda: # bound function iex&gt; Formular . eval ( &quot;1 + add.(-1, 5)&quot; , [ add : &amp; ( &amp;1 + &amp;2 ) ] ) { :ok , 5 } ... or with a context module: iex&gt; defmodule MyContext do ...&gt; def foo ( ) do ...&gt; 42 ...&gt; end ...&gt; end ...&gt; Formular . eval ( &quot;10 + foo&quot; , [ ] , context : MyContext ) { :ok , 52 } Directly calling to module functions in the code are disallowed for security reason. For example: iex&gt; Formular . eval ( &quot;Map.new&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;min(0, :os.system_time())&quot; , [ ] ) { :error , :no_calling_module_function } Evaluating AST instead of plain string code You may want to use AST instead of string for performance consideration. In this case, an AST can be passed to eval/3 : iex&gt; &quot;a = b = 10; a * b&quot; |&gt; Code . string_to_quoted! ( ) |&gt; Formular . eval ( [ ] ) { :ok , 100 } ...so that you don't have to parse it every time before evaluating it. Limiting execution time The execution time can be limited with the :timeout option: iex&gt; sleep = fn -&gt; :timer . sleep ( :infinity ) end ...&gt; Formular . eval ( &quot;sleep.()&quot; , [ sleep : sleep ] , timeout : 10 ) { :error , :timeout } Default timeout is 5_000 milliseconds. Limiting heap usage The evaluation can also be limited in heap size, with :max_heap_size option. When the limit is exceeded, an error {:error, :killed} will be returned. Example: iex&gt; code = &quot;for a &lt;- 0..999_999_999_999, do: to_string(a)&quot; ...&gt; Formular . eval ( code , [ ] , timeout : :infinity , max_heap_size : 1_000 ) { :error , :killed } The default max heap size is 1_000_000 words.","ref":"Formular.html","title":"Formular","type":"module"},{"doc":"Evaluate the code with binding context. Parameters code : code to eval. Could be a binary, or parsed AST. binding : the variable binding to support the evaluation options : current these options are supported: context : The module to import before evaluation. timeout : A timer used to terminate the evaluation after x milliseconds. 5000 milliseconds by default. max_heap_size : A limit on heap memory usage. If set to zero, the max heap size limit is disabled. 1000000 words by default. Examples iex&gt; Formular . eval ( &quot;1&quot; , [ ] ) { :ok , 1 } iex&gt; Formular . eval ( ~s[&quot;some text&quot;] , [ ] ) { :ok , &quot;some text&quot; } iex&gt; Formular . eval ( &quot;min(5, 100)&quot; , [ ] ) { :ok , 5 } iex&gt; Formular . eval ( &quot;max(5, 100)&quot; , [ ] ) { :ok , 100 } iex&gt; Formular . eval ( &quot;count * 5&quot; , [ count : 6 ] ) { :ok , 30 } iex&gt; Formular . eval ( &quot;add.(1, 2)&quot; , [ add : &amp; ( &amp;1 + &amp;2 ) ] ) { :ok , 3 } iex&gt; Formular . eval ( &quot;Map.new&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;Enum.count([1])&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;min(0, :os.system_time())&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;inspect.(System.A)&quot; , [ inspect : &amp; Kernel . inspect / 1 ] ) { :ok , &quot;System.A&quot; } iex&gt; Formular . eval &quot;f = &amp;IO.inspect/1&quot; , [ ] { :error , :no_calling_module_function } iex&gt; Formular . eval ( &quot;mod = IO; mod.inspect(1)&quot; , [ ] ) { :error , :no_calling_module_function } iex&gt; &quot;a = b = 10; a * b&quot; |&gt; Code . string_to_quoted! ( ) |&gt; Formular . eval ( [ ] ) { :ok , 100 }","ref":"Formular.html#eval/3","title":"Formular.eval/3","type":"function"},{"doc":"","ref":"Formular.html#t:code/0","title":"Formular.code/0","type":"type"},{"doc":"","ref":"Formular.html#t:eval_result/0","title":"Formular.eval_result/0","type":"type"},{"doc":"","ref":"Formular.html#t:option/0","title":"Formular.option/0","type":"type"},{"doc":"","ref":"Formular.html#t:options/0","title":"Formular.options/0","type":"type"}]